/*
 * Copyright (c) 2003, 2004, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package org.jboss.com.sun.corba.se.spi.orbutil.proxy;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.rmi.PortableRemoteObject;

import org.jboss.com.sun.corba.se.impl.logging.ORBUtilSystemException;
import org.jboss.com.sun.corba.se.spi.logging.CORBALogDomains;

@SuppressWarnings("serial")
public class CompositeInvocationHandlerImpl implements CompositeInvocationHandler
{
    private Map<Class<?>, InvocationHandler> classToInvocationHandler = new LinkedHashMap<Class<?>, InvocationHandler>();

    private InvocationHandler defaultHandler = null;

    public void addInvocationHandler(Class<?> interf, InvocationHandler handler)
    {
        classToInvocationHandler.put(interf, handler);
    }

    public void setDefaultHandler(InvocationHandler handler)
    {
        defaultHandler = handler;
    }

    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable
    {
        // Note that the declaring class in method is the interface in which the method was defined, not the proxy
        // class.
        Class<?> cls = method.getDeclaringClass();
        InvocationHandler handler = classToInvocationHandler.get(cls);

        if (handler == null)
        {
            if (defaultHandler != null)
                handler = defaultHandler;
            else
            {
                ORBUtilSystemException wrapper = ORBUtilSystemException.get(CORBALogDomains.UTIL);
                throw wrapper.noInvocationHandler("\"" + method.toString() + "\"");
            }
        }

        // handler should never be null here.

        Object result = handler.invoke(proxy, method, args);
        final Class<?> returnType = method.getReturnType();
        if(result == null) {
            return result;
        } else if(returnType.isPrimitive() || returnType.isAssignableFrom(result.getClass())) {
            return result;
        } else {
            return PortableRemoteObject.narrow(result, returnType);
        }
    }
}
